﻿#include "RealSense2CV.h"

// コンストラクタ
TRealSense2CV::TRealSense2CV()
{
}

// デストラクタ
TRealSense2CV::~TRealSense2CV()
{
  projection->Release();
  sm->Release();
}

// 初期化
bool TRealSense2CV::init()
{
  sm = PXCSenseManager::CreateInstance();

  //PXCSession::ImplDesc id = {};
  //pxcStatus sts = sm->QuerySession()->CreateImpl(&id, &capture);	

  sm->EnableStream(PXCCapture::STREAM_TYPE_COLOR);
  sm->EnableStream(PXCCapture::STREAM_TYPE_DEPTH);
  sm->EnableStream(PXCCapture::STREAM_TYPE_IR);
  Status stat = sm->Init();
  if (stat == PXC_STATUS_NO_ERROR) {
    device = sm->QueryCaptureManager()->QueryDevice();
    projection = device->CreateProjection();
    return true;
  }
  else return false;
}

// 画像を問い合わせ
bool TRealSense2CV::queryFrame()
{
  sm->ReleaseFrame();
  Status stat = sm->AcquireFrame();
  if (stat == PXC_STATUS_NO_ERROR) {
    sample = sm->QuerySample();
    return true;
  }
  else return false;
}

// カラー画像の取得
cv::Mat TRealSense2CV::color()
{
  colorMat = PXCcolor2Mat(sample->color, PXCImage::PIXEL_FORMAT_RGB24);
  return colorMat;
}

// 距離画像の取得
cv::Mat TRealSense2CV::depth()
{
  depth16Mat = PXCdepth162Mat(sample->depth, PXCImage::PIXEL_FORMAT_DEPTH);
  depth16Mat.convertTo(depthMat, CV_8UC1, -255.0 / 1200, 255);
  return depthMat;
}

// 距離画像（16bit）の取得
cv::Mat TRealSense2CV::depth16()
{
  return depth16Mat;
}

// 赤外線画像の取得
cv::Mat TRealSense2CV::ifred()
{
  ifredMat = PXCifred2Mat(sample->color, PXCImage::PIXEL_FORMAT_Y8);
  return ifredMat;
}

// カラー画像をMat型に変換
cv::Mat TRealSense2CV::PXCcolor2Mat(PXCImage *pxcImage, Intel::RealSense::Image::PixelFormat format)
{
  Intel::RealSense::ImageData data;
  pxcImage->AcquireAccess(PXCImage::ACCESS_READ, format, &data);
  cv::Mat img = cv::Mat(pxcImage->QueryInfo().height, pxcImage->QueryInfo().width, CV_8UC3, data.planes[0]);
  pxcImage->ReleaseAccess(&data);
  return img;
}

// 距離画像をMat型に変換
cv::Mat TRealSense2CV::PXCdepth162Mat(PXCImage *pxcImage, Intel::RealSense::Image::PixelFormat format)
{
  Intel::RealSense::ImageData data;
  pxcImage->AcquireAccess(PXCImage::ACCESS_READ, format, &data);
  cv::Mat img = cv::Mat(pxcImage->QueryInfo().height, pxcImage->QueryInfo().width, CV_16UC1, data.planes[0]);
  pxcImage->ReleaseAccess(&data);
  return img;
}

// 赤外線画像をMat型に変換
cv::Mat TRealSense2CV::PXCifred2Mat(PXCImage *pxcImage, Intel::RealSense::Image::PixelFormat format)
{
  Intel::RealSense::ImageData data;
  pxcImage->AcquireAccess(PXCImage::ACCESS_READ, format, &data);
  cv::Mat img = cv::Mat(pxcImage->QueryInfo().height, pxcImage->QueryInfo().width, CV_8UC1, data.planes[0]);
  pxcImage->ReleaseAccess(&data);
  return img;
}

// 距離画像からカメラ座標系への座標変換
std::vector<cv::Point3d> TRealSense2CV::depth2camera()
{
  std::vector<cv::Point3d> p;
  PXCPoint3DF32* vertices = new PXCPoint3DF32[depth16Mat.cols * depth16Mat.rows];
  projection->QueryVertices(sample->depth, vertices);

  for (int i = 0; i < depth16Mat.cols * depth16Mat.rows; i++) {
    p.push_back(cv::Point3d(vertices[i].x, vertices[i].y, vertices[i].z));
  }
  return p;
}
